@@ -465,49 +465,3 @@ vector<Node*> Node::getInnerNodes()
     }
 }
 
-
-void Node::deleteMe()
-{
-    Node* tmpSon ;
-    Node* tmpFather ;
-    if( isLeaf() )
-    {
-        if( isRoot() )
-        {
-            while (sons_.size()!=0)
-            {
-                remove(sons_.begin(), sons_.end() , sons_[0]);
-            }
-            distanceToFather_ = 0.0;
-            height_ = 0.0;
-            name_ = "";
-            isSampled_ = 0;
-        }
-        else
-        {
-            tmpFather = father_ ;
-            tmpFather->removeSon(this);
-            if( (tmpFather->sons_).size() == 1 )
-            {
-                tmpSon = (tmpFather->sons_)[0] ;
-                if( !(tmpFather->isRoot()) )
-                {
-                    tmpSon->setDistanceToFather(tmpSon->distanceToFather_ + tmpFather->distanceToFather_);
-                    tmpSon->setFather(tmpFather->father_);
-                    (tmpFather->father_)->removeSon(tmpFather);
-                }
-                else
-                {
-                    tmpFather->sons_ = tmpSon->sons_;
-                    for(unsigned i=0 ; i<(tmpFather->sons_).size() ; i++)
-                        ((tmpFather->sons_)[i])->father_ = tmpFather ;
-                    tmpFather->name_ = tmpSon->name_ ;
-                }
-            }
-        }
-    }
-    else
-    {
-        warning("Trying to remove an internal node.");
-    }
-}
